# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

#
# Snappy Ecosystem Tests
# Copyright (C) 2017 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

"""Web-Ui related functionality for testing web pages using selenium"""

import unittest

from selenium import webdriver
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC
from selenium.common.exceptions import TimeoutException
from selenium.webdriver.common.by import By

from snappy_ecosystem_tests import storeconfig

LOGIN_EMAIL, LOGIN_PASSWORD = storeconfig.get_store_credentials()
STORE_URL = storeconfig.get_store_web_url()

# default PAGE and ELEMENTS load timeout
PAGE_LOAD_TIMEOUT = 30


class UbuntuStoreWebTestsBase(unittest.TestCase):
    """Base class to represent a Web-UI test case.
    Contain basic functionality for locating web elements and interacting with a page
    """
    def setUp(self):
        # TODO - add support for multiple type browsers including Chrome, FF and IE
        options = webdriver.ChromeOptions()
        options.add_argument("--start-maximized")
        self.driver = webdriver.Chrome(chrome_options=options)

    def wait_for_element(self, locator, timeout=PAGE_LOAD_TIMEOUT):
        """Wait of element to be loaded  for the given timeout,
        default timeout is as per PAGE_LOAD_TIMEOUT variable"""
        wait = WebDriverWait(self.driver, timeout)
        try:
            element_present = EC.presence_of_element_located(locator)
            wait.until(element_present)
        except TimeoutException:
            print("Timed out waiting for "+locator[1]+" page")

    def wait_for_page_title(self, title, timeout=PAGE_LOAD_TIMEOUT):
        """Wait for the target page title for given timeout,
        default timeout is as per PAGE_LOAD_TIMEOUT variable"""
        wait = WebDriverWait(self.driver, timeout)
        try:
            find_title = EC.title_is(title)
            wait.until(find_title)
        except TimeoutException:
            print("Timed out waiting for "+title + " page")

    def wait_for_visible(self, timeout=PAGE_LOAD_TIMEOUT):
        # TODO: re-implement to allow any locator as parameter
        """Wait for the element to be visible for given timeout,
        default timeout is as per PAGE_LOAD_TIMEOUT variable"""
        wait = WebDriverWait(self.driver, timeout)
        try:
            element_visible = EC.visibility_of_element_located(
                (By.XPATH, "/html/body/nav/div/div[1]/span"))
            wait.until(element_visible)
        except TimeoutException:
            print("Timed out waiting for element to be visible")

    def wait_for_clickable(self, timeout=PAGE_LOAD_TIMEOUT):
        # TODO: re-implement to allow any locator as parameter
        """Wait for the element to be visible for given timeout,
        default timeout is as per PAGE_LOAD_TIMEOUT variable"""
        wait = WebDriverWait(self.driver, timeout)
        try:
            element_clickable = EC.element_to_be_clickable(
                (By.XPATH, "/html/body/nav/div/div[1]/span"))
            wait.until(element_clickable)
        except TimeoutException:
            print("Timed out waiting for element to be clickable")

    def login(self):
        """Login to web interface, following the UI flow which end user would use"""
        driver = self.driver
        driver.get(STORE_URL)

        signin_link = driver.find_element_by_link_text("Sign in or register")
        signin_link.click()
        self.wait_for_element((By.ID, 'login-form'))
        login_form = driver.find_element_by_id("login-form")
        email = login_form.find_element_by_id("id_email")
        email.send_keys(LOGIN_EMAIL)
        password = login_form.find_element_by_id("id_password")
        password.send_keys(LOGIN_PASSWORD)
        login_form.submit()
        self.wait_for_element((By.NAME, 'decideform'))
        decide_form = driver.find_element_by_name("decideform")
        decide_form.submit()
        self.wait_for_page_title("Your packages")

    def logout(self):
        """Logout of web interface, following the UI flow which end user would use"""
        driver = self.driver
        self.wait_for_element((By.CLASS_NAME, "b-dropdown__arrow_light"))
        #
        # extremely ugly way to reach the desired element by xpath, a bug will be
        # raised to uniquely identify the dropdown arrows
        self.wait_for_clickable()
        down_arrow = driver.find_element_by_xpath("/html/body/nav/div/div[1]/span")
        #
        # There is a bug with Firefox webdriver when clicking elements,
        # as sometime click doesn't have any effect, to workaround that
        # multiple attempts are made. However everything works fine with chrome webdriver.

        for _ in range(0, 2):
            wait = WebDriverWait(self.driver, 1)
            try:
                down_arrow.click()
                element_present = EC.presence_of_element_located((By.CLASS_NAME, "b-dropdown_open"))
                wait.until(element_present)
                break
            except TimeoutException:
                print("Failed to open the accounts dropdown menu card, Trying again")

        self.wait_for_visible()
        logout_link = driver.find_element_by_link_text("Log out")
        logout_link.click()

        self.wait_for_page_title("Sign in to see your packages")

    def tearDown(self):
        self.driver.close()
        self.driver.quit()
