# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

#
# Snappy Ecosystem Tests
# Copyright (C) 2017 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import getpass
import json
import os
import subprocess
import sys
import shlex
import shutil
import tempfile

import pexpect
import yaml

PATH_SNAP = '/usr/bin/snap'
COMMAND_DOWNLOAD = 'download {snap} --channel={channel}'
COMMAND_FIND = 'find {search_term}'
COMMAND_INFO = 'info {snap}'
COMMAND_INSTALL = 'install {snap} --channel={channel}'
COMMAND_LOGIN = 'login {email}'
COMMAND_LOGOUT = 'logout'
COMMAND_REFRESH = 'refresh {snap} --channel={channel}'
CHANNEL_STABLE = 'stable'
PASSWORD_ROOT = 'changed'


def is_root_user():
    """Return bool representing if the current user is root."""
    return os.getuid() == 0


def run_snapd_command(parameters, return_output=False, cwd=None):
    """ Run the request snapd cli command.

    :param parameters: the snapd sub-command and their parameters.
        example: install core --beta
    :param return_output: Whether to return the stdout of the command.
    :param cwd: The current working directory for the command, defaults to
        pwd.
    :return: Optionally return the stdout of the command, depending if
        the value of `return_output` was True.
    """
    command = '{} {}'.format(PATH_SNAP, parameters)
    if return_output:
        raw_output = subprocess.check_output(
            shlex.split(command), universal_newlines=True, cwd=cwd)
        return raw_output.strip().split('\n')
    subprocess.check_call(shlex.split(command), cwd=cwd)


def login(email, password):
    """Login to snapd.

    :param email: Ubuntu SSO account email address.
    :param password: Ubuntu SSO account password.
    """
    command = '{} {}'.format(PATH_SNAP, COMMAND_LOGIN.format(email=email))
    if is_root_user():
        p = pexpect.spawnu(command, logfile=sys.stdout)
    else:
        p = pexpect.spawnu('{} {}'.format('sudo', command), logfile=sys.stdout)
        p.expect_exact(
            '[sudo] password for {}: '.format(getpass.getuser()))
        p.sendline(PASSWORD_ROOT)
    p.expect_exact('Password of "{}": '.format(email))
    p.sendline(password)
    p.wait()
    assert _is_user_logged_in(email), '{} failed to login'.format(email)


def _is_user_logged_in(email):
    snapd_macaroon_path = os.path.expanduser('~/.snap/auth.json')
    if not os.path.exists(snapd_macaroon_path):
        return False
    with open(snapd_macaroon_path) as file:
        return json.loads(file.read()).get('email') == email


def logout():
    run_snapd_command(COMMAND_LOGOUT)


def download(snap, channel=CHANNEL_STABLE, path=None):
    """Download the requested snap.

    :param snap: name of the snap to download.
    :param channel: name of the release channel to download from.
    :param path: The path to which the downloaded snap be saved to, defaults
        to pwd.
    """
    command = COMMAND_DOWNLOAD.format(snap=snap, channel=channel)
    if path and os.path.isdir(path):
        with tempfile.TemporaryDirectory() as tempdir:
            run_snapd_command(command, cwd=tempdir)
            for file in os.listdir(tempdir):
                shutil.move(os.path.join(tempdir, file), path)
    else:
        run_snapd_command(command)


def install(snap, channel=CHANNEL_STABLE):
    """Install the requested snap."""
    run_snapd_command(COMMAND_INSTALL.format(snap=snap, channel=channel))


def info(snap, verbose=False):
    """Query the Ubuntu store of the information about a snap.

    :param snap: Name of the snap for which the info is required.
    :param verbose: Whether to information should be detailed.
    :return: Return a dictionary containing information about the snap.
    """
    command = COMMAND_INFO.format(snap=snap)
    if verbose:
        command = ' '.join([command, '--verbose'])
    output = run_snapd_command(command, return_output=True)
    return yaml.load("""{}""".format(output))


def refresh(snap, channel=CHANNEL_STABLE):
    """Refresh the requested snap."""
    run_snapd_command(COMMAND_REFRESH.format(snap=snap, channel=channel))


def find(keyword):
    """Find snaps based on the provided filters

    :param keyword: Keyword to use for the query.
    :return: Return a list of dictionaries containing information about
        snaps matching the `keyword`.
    """
    raw_output = run_snapd_command(
        COMMAND_FIND.format(search_term=keyword), return_output=True)
    headers = [header.lower() for header in raw_output.pop(0).split()]
    return [dict(zip(headers, line.split())) for line in raw_output]
